/*
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
* ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
* PARTICULAR PURPOSE.
*
* Copyright (C) 1996-1999  IDS Software.  All Rights Reserved.
*
*  Program: idsa.c  -- IDS Server Authentication Gateway
*
*           It was formerly known as IDS Server CGI Gateway.
*           The old name has created much confusion on the
*           purpose of this program, thus renamed in version
*           3.1.1 of IDS Server.
*
*           In short, "idsa" is a CGI program that allows an
*           IDS Server client to be authenticated by a full-
*           featured Web server before directly connecting to
*           the IDS Server. It is NOT a "tunneling" program 
*           that channels data to and from the IDS Server.
*           Please refer to Section 2.1.3 of the IDS Server 
*           User's Guide for more detail.
*
*  Compile: Modify the first two #define to setup the target
*           IDS Server host address and port number. Use the
*           following command line to compile on Solaris:
* 
*               cc idsa.c -o idsc -lsocket -lnsl -lc
*
*           For Windows, the program extracts the address and
*           the port number from WIN32 environment variable
*           IDSS_HOST and IDSS_PORT.  To compile on Windows:
*
*               cl /D WIN32 idsa.c /link advapi32.lib wsock32.lib
*/

#ifndef WIN32
#define IDSS_HOST	"localhost"
#define IDSS_PORT	12
#else
#define IDSS_HOST	GetHost()
#define IDSS_PORT	GetPort()
#endif

#ifdef WIN32
#include <winsock.h>
#include <tchar.h>
#include <time.h>
#include <process.h>
#else
#include <ctype.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#define closesocket	close
#endif

#include <stdlib.h>
#include <string.h>

/* Network order of adress 127.0.0.1 */
#define LOCAL_HOST	0x0100007fL

char szNoEnv[] =
"Environment variables IDSS_HOST and IDSS_PORT are not set. \
Contact your Web administrator.";
char szMethod[] =
"Unknown HTTP method. Only GET and POST are supported.";
char szNotHTTP[] =
"Unknown protocol. Only HTTP is supported.";
char szBad [] =
"Incorrect use of the POST method. If you login using a HTML \
&lt;FORM&gt;, use the GET method. If you are submitting a query, \
use the POST method and submit it directly to the IDS Server host.";

char szErrMsg[] =
"Content-type: text/html\r\nStatus: %s\r\n\r\n<html><head>\
<title>IDS Authentication Gateway Error</title></head><h3>\
<u>IDS Authentication Gateway Error</u></h3>%s<hr></html>";

void Error(int code, char *szMsg)
{
	char *status;

	switch (code) {
	default:
	case 400: status = "400 Bad Request"; break;
	case 500: status = "500 Internal Server Error"; break;
	case 501: status = "501 Not Implemented"; break;
	}
	printf(szErrMsg, status, szMsg);
	exit(code);
}


int GetEnv(char *env, char *value, int max)
{
#ifdef WIN32
	static char regSMEnv[] = "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment\\";
	HKEY	hKey;
	DWORD	size = max;
	long	stat = RegOpenKey(HKEY_LOCAL_MACHINE, regSMEnv, &hKey);

	if (stat == ERROR_SUCCESS) {
		stat = RegQueryValueEx(hKey, env, NULL, NULL, value, &size);
		RegCloseKey(hKey);
		if (stat == ERROR_SUCCESS)
			return TRUE;
	}
#endif
	if ((env = getenv(env)) != NULL && strlen(env) < max) {
		strcpy(value, env);
		return 1;
	}
	*value = 0;
	return 0;
}

char *GetHost()
{
	static char host[80];
	return GetEnv("IDSS_HOST", host, sizeof(host)) ? host : NULL;
}

int GetPort()
{
	char port[8];
	return GetEnv("IDSS_PORT", port, sizeof(port)) ? atoi(port) : -1;
}

int SocketConnect(char *host, int port)
{
	struct sockaddr_in sa_in;
	int    sock;

	sa_in.sin_family = AF_INET;
	sa_in.sin_port = htons((u_short)port);
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock != -1) {
		if (isdigit(*host))
			sa_in.sin_addr.s_addr = inet_addr(host);
		else {
			struct hostent	*p = gethostbyname(host);
			sa_in.sin_addr.s_addr = 
				(p != NULL ? *((u_long *)p->h_addr) : LOCAL_HOST);
		}
		if (connect(sock, (struct sockaddr *)&sa_in, sizeof(sa_in))) {
			closesocket(sock);
			return -1;
		}
	}
	return sock;
}

int SocketSend(int sock, char *buf, int len)
{
	int	n;
	while (len && (n = send(sock, buf, len, 0)) != len) {
		if (n == -1)
			return -1;
		buf += n;
		len -= n;
	}
	return 0;
}

int SocketRecv(int sock, char *buf, int max)
{
	return recv(sock, buf, max, 0);
}

int main(int argc, char *argv[])
{
	int 	sock;
	char	buf[1024];
	int		port, n, len, size;
	char	*host, *env, *protocol;
	int 	bFixHeader = 1;
#ifdef WIN32
	WSADATA		WSAData;
	if (WSAStartup(0x0101, &WSAData))
		Error(500, "WinSock error");
#endif

	host = IDSS_HOST;
	port = IDSS_PORT;
	if (host == NULL || port < 0)
		Error(500, szNoEnv);

	env = getenv("REQUEST_METHOD");
	if (env == NULL)
		Error(400, szMethod);
	if (strcmp(env, "POST") == 0)
		Error(400, szBad);
	if (strcmp(env, "GET"))
		Error(501, szMethod);

	protocol = getenv("SERVER_PROTOCOL");
	if (strstr(protocol, "HTTP/") != protocol)
		Error(501, szNotHTTP);

	strcpy(buf, "HEAD ");
	strcat(buf, getenv("PATH_INFO"));
	strcat(buf, "?");
	strcat(buf, getenv("QUERY_STRING"));
	strcat(buf, " ");
	strcat(buf, protocol);
	strcat(buf, "\r\nIdss-client: "); /* IDSS specific header */
	strcat(buf, getenv("REMOTE_ADDR"));
	strcat(buf, "\r\nReferer: http://");
	strcat(buf, getenv("SERVER_NAME"));
	strcat(buf, ":");
	strcat(buf, getenv("SERVER_PORT"));
	strcat(buf, "/\r\n\r\n");

	sock = SocketConnect(host, port);
	if (sock == -1)
		Error(500, "Socket connect error");
	len = SocketSend(sock, buf, strlen(buf));
	if (len != 0)
		Error(500, "Socket send error");
	size = sizeof(buf);
	do {
		n = SocketRecv(sock, buf + len, size - len - 1);
		if (n == -1)
			Error(500, "Socket read error");
		buf[len += n] = 0;
		if (bFixHeader && (protocol = strchr(buf, ' ')) != NULL) {
			if (protocol - buf < 7)
				Error(501, szNotHTTP);
			strcpy(buf, "Status:");
			strcpy(buf+7, protocol);
			bFixHeader = 0;
			len = strlen(buf);
		}
		if (size - len < 8 || !n) {
			printf("%s", buf);
			len = 0;
		}
	} while (n);

	closesocket(sock);
	return 0;
}

