#!/usr/local/bin/perl
##
# IDS Server Authentication Gateway Perl Script
#
# This script requires Perl 5.002 or higher.  It also require network
# Perl module IO::Socket, which can only come from a fully installed
# Perl and not just a copy of perl.exe.  For Windows NT/95 users, you
# can get the latest Perl for Win32 from ActiveWare Internet Corp. at
# http://www.ActiveWare.com/
#
# THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
# ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
# PARTICULAR PURPOSE.
#
require 5.002;
use strict;
use IO::Socket;

my ($host, $port, $sock, $line);
sub error;

##
# Extract IDS Server address and port number from environment 
# variable IDSS_HOST and IDSS_PORT.  You can also hard-code 
# these two values for simplicity.  Netscape web servers do
# NOT pass environment variables to CGI programs, so you must
# assgin these two values in the script.
#
$host = $ENV{'IDSS_HOST'};
$port = $ENV{'IDSS_PORT'};

if (!defined($host) || !defined($port)) {
  error "Environment variables IDSS_HOST and IDSS_PORT are not set.";
}

##
# Create a socket to the IDS Server
#
$sock = IO::Socket::INET->new(PeerAddr => $host,
                              PeerPort => "http($port)",
                              Proto    => 'tcp');
if (!defined($sock)) {
  error "Authentication Gateway cannot connect to IDS Server";
}
$sock->autoflush(1);

##
# Send request to IDS Server
#
print $sock "HEAD $ENV{'PATH_INFO'}?$ENV{'QUERY_STRING'} $ENV{'SERVER_PROTOCOL'}\n";
print $sock "IDSS-Client: $ENV{'REMOTE_ADDR'}\n";
print $sock "Referer: http://$ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}/\n";
print $sock "\n";

##
# Read status line from IDS Server
#
$line = $sock->getline();

##
# Change status line from "HTTP/1.0 " into "Status: "
#
$line =~ /[^ ]* ([^\r\n]*)/;
print "Status: $1\n";

##
# Relay all remaining data back to client
#
binmode(STDOUT);
while ($sock->read($line, 1024) != undef) {
  print $line;
}

##
# Close socket and done
#
close($sock);
exit;

##
# Error reply subroutine (dies)
#
sub error { #(msg)
  my($msg) = @_;
  print "Status: 404 $msg\nContent-type: text/html\n\n";
  print "<html><head><title>IDS Server Authentication Gateway Error</title></head>";
  print "<h3><u>IDS Server Authentication Gateway Error</u></h3> $msg <hr></html>";
  die "$msg ($!)";
}
