/*
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
* ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
* PARTICULAR PURPOSE.
*
* Copyright (C) 1996-2003  IDS Software.  All Rights Reserved.
*
*  Program: idsx.c  -- IDS Server Proxy
*
*           This Web server program (DLL or SO) creates a 
*           persistent socket connection for a client session
*           to the target IDS Server. It can be compiled as
*           an ISAPI, NSAPI or Apache API application for the
*           respective Web server. The target IDS Server can
*           be running on the same machine as the Web server,
*           another machine on the internet or one inside the
*           firewall, as long as it can be reached from the Web
*           server machine. This program basically turns the
*           Web server into a proxy server between its clients
*           and the target IDS Server.
*
*  Compile: Define symbol DO_ISAPI, DO_NSAPI or DO_APAPI when
*           compiling to create ISAPI, NSAPI and Apache module
*           respectively.
*
*           By default, the program obtain the IDS Server host
*           address and port number from environment variable
*           "IDSS_HOST" and "IDSS_PORT". You may override and
*           hard-code the address and port number by defining
*           symbol IDSS_HOST and IDSS_PORT at compile time.
*/

#ifndef IDSS_HOST
#define IDSS_HOST	GetHost()
#endif
#ifndef IDSS_PORT
#define IDSS_PORT	GetPort()
#endif

#define IOBUF_SIZE	2048

#ifdef WIN32
#ifdef DO_APAPI
# include <ap_config.h>
#else
# include <winsock.h>
#endif
#include <tchar.h>
#include <process.h>
#else
#include <ctype.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#define closesocket	close
#define stricmp strcasecmp
#endif

#include <stdlib.h>
#include <string.h>

/* Network order of adress 127.0.0.1 */
#define LOCAL_HOST	0x0100007fL

#ifdef WIN32
char *GetEnv(char *env)
{
	static char regSMEnv[] = "SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment\\";
	HKEY	hKey;
	DWORD	size = 0;
	char*	value= getenv(env);
	LONG	stat = RegOpenKey(HKEY_LOCAL_MACHINE, regSMEnv, &hKey);
	int		len;

	if (stat == ERROR_SUCCESS && value == NULL) {
		stat = RegQueryValueEx(hKey, env, NULL, NULL, NULL, &size);
		if (stat == ERROR_SUCCESS) {
			value = (char*)malloc(size + (len = strlen(env)) + 1);
			if (value != NULL) {
				strcpy(value, env);
				value[len] = '=';
				RegQueryValueEx(hKey, env, NULL, NULL, (LPBYTE)&value[len+1], &size);
				_putenv(value);
				free(value);
				value = getenv(env);
			}
		}
		RegCloseKey(hKey);
	}
	return value;
}
#else
#define GetEnv(env) getenv(env)
#endif

char *GetHost()
{
	return GetEnv("IDSS_HOST");
}

int GetPort()
{
	char *port = GetEnv("IDSS_PORT");
	return (port != NULL ? atoi(port) : -1);
}

int SocketConnect(char *host, int port)
{
	struct sockaddr_in sa_in;
	int    sock;

	sa_in.sin_family = AF_INET;
	sa_in.sin_port = htons((u_short)port);
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock != -1) {
		if (isdigit(*host))
			sa_in.sin_addr.s_addr = inet_addr(host);
		else {
			struct hostent	*p = gethostbyname(host);
			sa_in.sin_addr.s_addr = 
				(p != NULL ? *((u_long *)p->h_addr) : LOCAL_HOST);
		}
		if (connect(sock, (struct sockaddr *)&sa_in, sizeof(sa_in))) {
			closesocket(sock);
			return -1;
		}
	}
	return sock;
}

int SocketSend(int sock, char *buf, int len)
{
	int	n;
	while (len && (n = send(sock, buf, len, 0)) != len) {
		if (n == -1)
			return -1;
		buf += n;
		len -= n;
	}
	return 0;
}

int SocketRecv(int sock, char *buf, int max)
{
	return recv(sock, buf, max, 0);
}

int strfnd(char *string, char *target)
{
	char *p = strstr(string, target);
	return (p == NULL ? -1 : p - string);
}

struct _SocketBuffer {
	int		sock;
	int		size;
	int		oPos;
	char*	oBuf;
	void*	ctx;
};
typedef struct _SocketBuffer SocketBuffer;

void SocketProxy(SocketBuffer *sb, int sock)
{
	fd_set	fds;
	int		i, n = (sb->sock > sock ? sb->sock : sock) + 1;

	while (1) {

		FD_ZERO(&fds);
		FD_SET(sock, &fds);
		FD_SET(sb->sock, &fds);

		i = select(n, &fds, NULL, NULL, NULL);

		if (i <= 0)
			break;
		if (FD_ISSET(sock, &fds)) {
			if ((i = SocketRecv(sock, sb->oBuf, sb->size)) <= 0)
				break;
			if (SocketSend(sb->sock, sb->oBuf, i))
				break;
		}
		else if (FD_ISSET(sb->sock, &fds)) {
			if ((i = SocketRecv(sb->sock, sb->oBuf, sb->size)) <= 0)
				break;
			if (SocketSend(sock, sb->oBuf, i))
				break;
		}
		else
			break;
	}
}

int Connect(SocketBuffer *sb, char *host, int port)
{
	memset(sb, 0, sizeof(SocketBuffer));
	sb->sock = SocketConnect(host, port);
	if (sb->sock != -1) {
		sb->size = IOBUF_SIZE;
		sb->oBuf = (char*)malloc(sb->size);
	}
	return sb->sock != -1 && sb->oBuf != NULL;
}

void Close(SocketBuffer *sb)
{
	if (sb->sock != -1)
		closesocket(sb->sock);
	if (sb->oBuf != NULL)
		free(sb->oBuf);
}

void Flush(SocketBuffer *sb)
{
	SocketSend(sb->sock, sb->oBuf, sb->oPos);
	sb->oPos = 0;
}

void Send(SocketBuffer *sb, const char *buf, int len)
{
	int n;
	while (len >= (n = sb->size - sb->oPos)) {
		memcpy(sb->oBuf + sb->oPos, buf, n);
		sb->oPos += n;
		buf += n;
		len -= n;
		Flush(sb);
	}
	if (len > 0) {
		memcpy(sb->oBuf + sb->oPos, buf, len);
		sb->oPos += len;
	}
}

void SendStr(SocketBuffer *sb, const char *str)
{
	Send(sb, str, strlen(str));
}

void SendHeader(SocketBuffer *sb, char *name, char *value)
{
	int is_conn = !stricmp(name, "Connection");

	SendStr(sb, name); Send(sb, ": ", 2);
	/*
	* Prevent IDS Server from handling keep-alive
	*/
	if (is_conn)
		Send(sb, "close\n", 6);
	else {
		SendStr(sb, value);
		Send(sb, "\n", 1);
	}
}

int FindServer(char *host[])
{
	int	port = IDSS_PORT;
	*host = IDSS_HOST;
	return (*host == NULL || port < 0 ? -1 : port);
}

#ifdef DO_ISAPI

#include <httpext.h>

typedef void (*ThreadFunc)(void *);

int ISAPI_send(EXTENSION_CONTROL_BLOCK *ecb, char *buf, int len)
{
	DWORD n, save;
	n = save = len;
	while (len) {
		if (!(*ecb->WriteClient)(
			ecb->ConnID, //HCONN ConnID, 
			buf, //LPVOID Buffer, 
			&n,  //LPDWORD lpdwBytes, 
			HSE_IO_SYNC)) //DWORD dwReserved 
			return SOCKET_ERROR;
		buf += n;
		n = (len -= n);
	}
	return save;
}

int ISAPI_recv(EXTENSION_CONTROL_BLOCK *ecb, char *buf, int len)
{
	if (!(*ecb->ReadClient)(
		ecb->ConnID, //HCONN hConn, 
		buf, //LPVOID lpvBuffer, 
		(LPDWORD)&len)) //LPDWORD lpdwSize 
		return SOCKET_ERROR;
	return len;
}

void ISAPI_relay(SocketBuffer *sb)
{
	int		len;
	char	buf[IOBUF_SIZE];

	while (
		(len = SocketRecv(sb->sock, buf, IOBUF_SIZE)) > 0 &&
		ISAPI_send((EXTENSION_CONTROL_BLOCK *)sb->ctx, buf, len) == len
	);
}

BOOL WINAPI GetExtensionVersion( HSE_VERSION_INFO  *pVer )
{
	WSADATA	WSAData;
	if (WSAStartup(0x0101, &WSAData))
		return FALSE;

    pVer->dwExtensionVersion = MAKELONG( HSE_VERSION_MINOR,
                                         HSE_VERSION_MAJOR );
    lstrcpyn( pVer->lpszExtensionDesc,
              "IDS Server Proxy 1.0 for IIS", HSE_MAX_EXT_DLL_NAME_LEN );

	return TRUE;
}
 
DWORD WINAPI HttpExtensionProc( EXTENSION_CONTROL_BLOCK  *ecb )
{
	SocketBuffer sb;
	char	*p, *s, *t;
	int		i, n = FindServer(&p);
	BOOL	iis4 = TRUE;

	if (n == -1) {
		ecb->dwHttpStatusCode = 500;
		return HSE_STATUS_ERROR;
	}
	if (!Connect(&sb, p, n)) {
		ecb->dwHttpStatusCode = 503;
		return HSE_STATUS_ERROR;
	}
	/*
	* Reconstruct request line
	*/
	SendStr(&sb, ecb->lpszMethod); Send(&sb, " ", 1);
	SendStr(&sb, ecb->lpszPathInfo);
	if (*ecb->lpszQueryString) {
		Send(&sb, "?", 1);
		SendStr(&sb, ecb->lpszQueryString);
	}
	Send(&sb, " HTTP/1.0\n", 10);
	/*
	* Reconstruct all HTTP headers
	*/
	p = sb.oBuf + (i = sb.oPos);
	n = sb.size - i;
	s = "ALL_RAW";
	while (!(*ecb->GetServerVariable)(ecb->ConnID, s, p, &n)) {
		switch (GetLastError()) {
		case ERROR_INVALID_INDEX:
			s = "ALL_HTTP";
			iis4 = FALSE;
			break;
		case ERROR_INSUFFICIENT_BUFFER:
			p = (char*)malloc(sb.size = i + n);
			if (p != NULL) {
				memcpy(p, sb.oBuf, i);
				free(sb.oBuf);
				sb.oBuf = p;
				p += i;
				break;
			}
		default:
			Close(&sb);
			ecb->dwHttpStatusCode = 501;
			return HSE_STATUS_ERROR;
		}
	}
	if (iis4)
		sb.oPos += n - 1;
	else {
	  for (; *p; ++p) {
		s = (p += 5); // Skip prefix "HTTP_"
		/*
		* Fix ISAPI using underscore for headers
		*/
		for (; (i = *p) && i != ':' && i != '\n'; ++p) {
			if (i == '_') *p = '-';
		}
		p = strchr(t = p, '\n');
		if (p == NULL || i != ':')
			break;
		*t = *p = '\0';
		SendHeader(&sb, s, t+1);
	  }
	}
	Send(&sb, "\n", 1);
	/*
	* Additional data (entity body) already read by ISAPI
	*/
	Send(&sb, (char*)ecb->lpbData, ecb->cbAvailable);
	Flush(&sb);

	sb.ctx = ecb;
	i = _beginthread((ThreadFunc)ISAPI_relay, 0, (void *)&sb);

	while (WaitForSingleObject((HANDLE)i, 0) == WAIT_TIMEOUT) {
		n = ISAPI_recv(ecb, sb.oBuf, sb.size);
		if ((n > 0 && SocketSend(sb.sock, sb.oBuf, n) == 0) ||
		    (n < 0 && GetLastError() == WSAETIMEDOUT))
			continue;
		else {
			closesocket(sb.sock);
			sb.sock = -1;
			if (WaitForSingleObject((HANDLE)i, 5000) == WSAETIMEDOUT)
				CloseHandle((HANDLE)i);
		}
	}
	Close(&sb);

	return HSE_STATUS_SUCCESS_AND_KEEP_CONN;
}

BOOL WINAPI TerminateExtension( DWORD dwFlags )
{
	if (dwFlags & HSE_TERM_MUST_UNLOAD) {
		WSACleanup();
		return TRUE;
	}
	return FALSE;
}

#endif // DO_ISAPI

#ifdef DO_NSAPI

#ifdef WIN32
#define XP_WIN32
#else
#define XP_UNIX
#endif
#define MCC_HTTPD

#include "base/pblock.h"
#include "base/session.h"
#include "frame/req.h"
#include "frame/http.h"

NSAPI_PUBLIC int IDSServerProxyProc(pblock *pb, Session *sn, Request *rq)
{
	struct pb_entry **ht = rq->headers->ht;
	struct pb_entry *e;
	netbuf *b = sn->inbuf;
	SocketBuffer sb;
	char	*from = pblock_findval("from", pb);
	char	*p;
	int		i, n = FindServer(&p);

	if (n == -1) {
		http_status(sn, rq, 500, NULL);
		return REQ_ABORTED;
	}
	if (!Connect(&sb, p, n)) {
		http_status(sn, rq, 503, NULL);
		return REQ_ABORTED;
	}
	/*
	* Reconstruct request line
	*/
	SendStr(&sb, pblock_findval("method", rq->reqpb)); Send(&sb, " ", 1);
	p = pblock_findval("uri", rq->reqpb);
	if (from != NULL) {
		n = strlen(from);
		if (from[n-1] == '*')
			from[--n] = '\0'; // Ignore trailing '*'
		i = strfnd(p, from);
		if (from[n-1] == '/') // Don't delete trailing '/'
			--n;
		if (i >= 0)
			p += n + i;
	}
	SendStr(&sb, p);
	p = pblock_findval("query", rq->reqpb);
	if (p != NULL && *p) {
		Send(&sb, "?", 1);
		SendStr(&sb, p);
	}
	Send(&sb, " HTTP/1.0\n", 10);
	/*
	* Reconstruct all HTTP headers
	*/
	n = rq->headers->hsize;
	for (i = 0; i < n; ++i) {
		if ((e = ht[i]) != NULL)
		  do {
			  SendHeader(&sb, e->param->name, e->param->value);
		  } while ((e = e->next) != NULL);
	}
	Send(&sb, "\n", 1);
	/*
	* Additional data (entity body) already read by NSAPI
	*/
	Send(&sb, (char*)b->inbuf + b->pos, b->cursize - b->pos);
	Flush(&sb);

	SocketProxy(&sb, *((int*)sn->csd)); /* Shhhhhh */

	Close(&sb);

	return REQ_PROCEED;
}

#endif // DO_NSAPI

#ifdef DO_APAPI

#undef closesocket

#include <httpd.h>
#include <http_config.h>

module MODULE_VAR_EXPORT idsx_module;
int idsx_handler(request_rec *r);

handler_rec idsx_handlers[] = {
	{ "ids-server-proxy", idsx_handler },
	{ NULL, NULL}
};

module MODULE_VAR_EXPORT idsx_module = {
   STANDARD_MODULE_STUFF,
   NULL,			/* initializer */
   NULL,			/* create per-dir config */
   NULL,			/* merge per-dir config */
   NULL,			/* server config */
   NULL,			/* merge server config */
   NULL,			/* command table */
   idsx_handlers,	/* handlers */
   NULL,			/* filename translation */
   NULL,			/* check_user_id */
   NULL,			/* check auth */
   NULL,			/* check access */
   NULL,			/* type_checker */
   NULL,			/* logger */
   NULL				/* header parser */
};

int ap_FindServer(const table *env, char *host[])
{
	int port = FindServer(host);
	if (port == -1) {
		char *p = (char*)ap_table_get(env, "IDSS_PORT");
		if (p != NULL)
			port = atoi(p);
		*host = (char*)ap_table_get(env, "IDSS_HOST");
		return (*host == NULL || port < 0 ? -1 : port);
	}
	return port;
}

int idsx_handler(request_rec *r)
{
    array_header *arr = ap_table_elts(r->headers_in);
    table_entry *elts = (table_entry *)arr->elts;
	SocketBuffer sb;
	BUFF	*buff = r->connection->client;
	char	*pref = (char*)ap_table_get(r->subprocess_env, "IDSS_PREFIX");
	char	*p;
	int		i, n = ap_FindServer(r->subprocess_env, &p);

	if (n == -1) {
		return 500;
	}
	if (!Connect(&sb, p, n)) {
		return 503;
	}
	/*
	* Reconstruct request line
	*/
	SendStr(&sb, r->method); Send(&sb, " ", 1);
	p = r->uri;
	if (pref != NULL) {
		n = strlen(pref);
		i = strfnd(p, pref);
		if (pref[n-1] == '/') // Don't delete trailing '/'
			--n;
		if (i >= 0)
			p += n + i;
	}
	SendStr(&sb, p);
	p = r->args;
	if (p != NULL && *p) {
		Send(&sb, "?", 1);
		SendStr(&sb, p);
	}
	Send(&sb, " ", 1); SendStr(&sb, r->protocol); Send(&sb, "\n", 1);
	/*
	* Reconstruct all HTTP headers
	*/
    for (i = 0; i < arr->nelts; ++i) {
		SendHeader(&sb, elts[i].key, elts[i].val);
	}
	Send(&sb, "\n", 1);
	/*
	* Additional data (entity body) already read by Apache
	*/
	Send(&sb, (char*)buff->inptr, buff->incnt);
	Flush(&sb);

	SocketProxy(&sb, buff->fd);

	Close(&sb);

	return OK;
}

#endif // DO_APAPI

#ifdef DO_APAPI2

#undef closesocket

#include <httpd.h>
#include <http_log.h>
#include <http_config.h>
#include <http_protocol.h>
int  idsx_handler(request_rec *r);
void register_hooks(apr_pool_t *p);

static void register_hooks(apr_pool_t *p)
{
    ap_hook_handler(idsx_handler, NULL, NULL, APR_HOOK_MIDDLE);
}

module AP_MODULE_DECLARE_DATA idsx_module =
{
    STANDARD20_MODULE_STUFF,
    NULL,                     /* create per-directory config structure */
    NULL,                     /* merge per-directory config structures */
    NULL,                     /* create per-server config structure */
    NULL,                     /* merge per-server config structures */
    NULL,                     /* command handlers */
    register_hooks            /* register hooks */
};

int ap_FindServer(const apr_table_t *env, char *host[])
{
	int port = FindServer(host);
	if (port == -1) {
		char *p = (char*)apr_table_get(env, "IDSS_PORT");
		if (p != NULL)
			port = atoi(p);
		*host = (char*)apr_table_get(env, "IDSS_HOST");
		return (*host == NULL || port < 0 ? -1 : port);
	}
	return port;
}

#ifdef WIN32
#include "arch/win32/apr_arch_networkio.h"
#else
#include "arch/unix/apr_arch_networkio.h"
#endif
int ap_FindSocket(conn_rec *conn)
{
	ap_filter_t  *f = conn->input_filters;
	core_net_rec *net;
	apr_socket_t *sock;

	// Look for the "core_in" filter
	for (sock = NULL; f != NULL; f = f->next) {
		net = (core_net_rec *)f->ctx;
		if (!strcmp(f->frec->name, "core_in") && net && net->c == conn)
			sock = net->client_socket;
	}
	if (sock == NULL)
		return -1;
	return (int)sock->socketdes;
}

int idsx_handler(request_rec *r)
{
    const apr_array_header_t *arr = apr_table_elts(r->headers_in);
    const apr_table_entry_t *elts = (const apr_table_entry_t *)arr->elts;
	SocketBuffer sb;
	char	*pref = (char*)apr_table_get(r->subprocess_env, "IDSS_PREFIX");
	char	*p;
	int		i, s, n = ap_FindServer(r->subprocess_env, &p);

    if (strcmp(r->handler, "ids-server-proxy")) {
        return DECLINED;
	}
	s = ap_FindSocket(r->connection);
	if (n == -1 || s == -1) {
		return 500;
	}
	if (!Connect(&sb, p, n)) {
		return 503;
	}
	/*
	* Reconstruct request line
	*/
	SendStr(&sb, r->method); Send(&sb, " ", 1);
	p = r->uri;
	if (pref != NULL) {
		n = strlen(pref);
		i = strfnd(p, pref);
		if (pref[n-1] == '/') // Don't delete trailing '/'
			--n;
		if (i >= 0)
			p += n + i;
	}
	SendStr(&sb, p);
	p = r->args;
	if (p != NULL && *p) {
		Send(&sb, "?", 1);
		SendStr(&sb, p);
	}
	Send(&sb, " ", 1); SendStr(&sb, r->protocol); Send(&sb, "\n", 1);
	/*
	* Reconstruct all HTTP headers
	*/
    for (i = 0; i < arr->nelts; ++i) {
		SendHeader(&sb, elts[i].key, elts[i].val);
	}
	Send(&sb, "\n", 1);
	Flush(&sb);

	SocketProxy(&sb, s);

	Close(&sb);

	return OK;
}

#endif // DO_APAPI2
