/*
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
* ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
* PARTICULAR PURPOSE.
*
* Copyright (C) 1997-2003  IDS Software.  All Rights Reserved.
*
*  Module:	AppTest.java
*
*  Comment:	IDS JDBC Driver - Applet Zero Installation Test Program
*
*		This applet interactively obtains the hostname (or address)
*		of the IDS Server before running the test queries. You 
*		don't need to modify and recompile the program at all.
*
*		To run the AppTest applet from a remote browser, type in 
*		the following URL:
*
*			http://server1.bogus.com:12/AppTest.htm
*			http://server1.bogus.com:12/ProxyTest.htm
*
*		The second URL launch this applet with a digital signature
*		to demonstrate the use of the ProxyProperties class.  For two
*		informative articles regarding firewall access in applets,
*		please visit the following Web pages on our Web site:
*
*			http://www.idssoftware.com/jdbchttps.html
*			http://www.idssoftware.com/fwapplet.html
*
*		If you are running the applet locally or your computer has 
*		no network connection, type in the following URL:
*
*			http://127.0.0.1:12/AppTest.htm
*			http://127.0.0.1:12/ProxyTest.htm
*
* [History]
* 01-03-03  Added the use of dbShield.
* 01-02-02  Changed to use JDK 1.1. No longer works in 3.x browsers.
* 12-02-99  Changed pxProp to Properties so this works in 3.x browsers.
* 11-01-99  Demonstrate using the ProxyProperties class.
* 08-30-99  Removed the meaningless nativeSQL() call.
* 12-10-97  Added proxy server access controls for Version 2.5
* 07-25-97  Added getHost() method to automatically extract hostname.
* 05-12-97  Changed into an interactive applet to obtain hostname.
* 03-07-97  Changed data source and queries to 'IDSExamples'
* 02-25-97  Added exception display to the catch block of method run().
* 02-12-97  Modified to bypass DriverManager to avoid problems with IE.
*
*/

import java.util.Properties;
import java.net.URL;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;
import netscape.security.*;

public class AppTest extends Applet 
	implements Runnable, ActionListener, ItemListener
{
	static String urlPrefix = "jdbc:ids://";
	static String conn_etc  = "/conn?dsn=IDSExamples";

	static String query[] = {
		"<sql name=departments>", // dbShield query
		"select * from courses",  // Matches query <sql name=courses>
		"drop table course",      // dbShield will deny this
		"<sql name=my_courses>",  // dbShield query
	};

	private TextArea textWin = null;
	private Thread theThread = null;
	private TextField status = new TextField("Please wait...");
	private TextField   host = new TextField();
	private Label     prompt = new Label("Enter IDS Server address: ");
	private Button runButton = new Button(" Run ");
	private boolean firstRun = true;
	private Checkbox   proxy = new Checkbox("Use proxy server:");
	private TextField pxHost = new TextField();
	private Label     lb_uid = new Label(" Proxy username:");
	private TextField px_uid = new TextField();
	private Label     lb_pwd = new Label(" Proxy password:");
	private TextField px_pwd = new TextField();
	private Choice    pxType = new Choice();

	private Properties pxProp = null;
   
	public void init() { // Applet initialization
		super.init();
		if (textWin == null) {
			Panel top, p1, p2;
			// Create a non-proportional font
			Font fn = new Font("Courier", Font.PLAIN, 12);
			// Create the output text window
			textWin = new TextArea();
			// Assign the fixed font to the output window
			textWin.setFont(fn);
			// Add the 3 types of proxy server to the list
			pxType.addItem("SOCKS v4 Server"); // 0
			pxType.addItem("SOCKS v5 Server"); // 1
			pxType.addItem("SSL Tunnelling");  // 2
			pxType.select(2);
			// Let newPanel() do all the layout work
			p1 = newPanel(prompt, null, host, null, runButton);
			p2 = newPanel(proxy, null, pxHost, null, pxType);
			p1 = newPanel(null, p1, null, p2, null);
			p2 = newPanel(lb_uid, null, px_uid, null, null);
			p2 = newPanel(p2, null, null, null, 
			     newPanel(lb_pwd, null, px_pwd, null, null));
			// Make the top group
			top = newPanel(null, null, p1, p2, null);
			// Disable most proxy server related controls
			pxHost.setEnabled(false); pxType.setEnabled(false);
			lb_uid.setEnabled(false); px_uid.setEnabled(false);
			lb_pwd.setEnabled(false); px_pwd.setEnabled(false);
			// Assign a layout manager for the main group
			setLayout(new BorderLayout());
			// Add components into the main group
			add("North", top);
			add("Center", textWin);
			add("South", status);
			// Enable proxy connection only if we are signed
			try {
				// Create a ProxyProperties instance on success
				pxProp = new ProxyProperties();
				try {
					// Netscape needs this even if the applet is signed
					PrivilegeManager.enablePrivilege("UniversalPropertyRead");
				}
				catch (Throwable e) {}
				// This call will succeed only if the applet is signed
				System.getProperty("java.home");
			}
			catch (SecurityException e) {
				// Applet is not signed with a digital signature
				proxy.setEnabled(false);
			}
			catch (Throwable e) {
				// Most likely a JDK 1.0.2 browser (IE or Nav 3.x)
				proxy.setEnabled(false);
			}
			runButton.addActionListener(this);
			proxy.addItemListener(this);
			pxType.addItemListener(this);
		}
	}

	public void start() {
		textWin.setText("");
		if (theThread == null) {
			theThread = new Thread(this);
			theThread.start();
		}
	}
	public void stop() {
		if ((theThread != null) && theThread.isAlive()) 
			theThread.stop();
		theThread = null;
	}
    
	public void run() {

	  try {
		/*
		* Let's first compose the connection URL
		*/
		String url  = urlPrefix;     // Always starts with this
		String addr = host.getText();
		if (addr.length() == 0) {    // Do nothing if no address
			status.setText("Ready");
			if (firstRun) {
				// Fill in the host name field
				host.setText(getHost());
				firstRun = false;
			}
			return;
		}
		url += addr + conn_etc;      // Compose the connection URL
		/*
		* Create an IDS JDBC Driver of the ids.sql package
		*/
		status.setText("Initializing IDS JDBC Driver...");
		Driver drv = new ids.sql.IDSDriver();
		/*
		* Gather proxy server settings if specified
		*/
		Properties info = null;
		if (proxy.getState()) {
			info = new Properties();
			info.put("proxy_host", pxHost.getText());
			int i = pxType.getSelectedIndex();
			info.put("proxy_type", i < 1 ? "1" : (i < 2 ? "2" : "4"));
			info.put("proxy_uid", px_uid.getText());
			info.put("proxy_pwd", px_pwd.getText());
			try {
				// Netscape needs this even if the applet is signed
				PrivilegeManager.enablePrivilege("UniversalConnect");
			}
			catch (Throwable e) {}
		}
		/*
		* Connect to the IDS Server using the composed URL
		*/
		status.setText("Connecting to IDS Server...");
		Connection conn = drv.connect(url, info);

		ResultSet rs;
		int n;
		/*
		* Create a dummy Statement class instance
		*/
		status.setText("Creating SQL statement object...");
		Statement stmt = conn.createStatement();
		/*
		* Loop through the first three queries in the query[] array.
		* Note that the same Statement instance is re-used during
		* each iteration.
		*/
		status.setText("Executing SQL statements...");
		for (n = 0; n < 3; ++n) {
			try {
				println();
				println(query[n]);
				// Execute the predefined query in the array
				rs = stmt.executeQuery(query[n]);
				// Print out the query result
				printQuery(rs);
			} catch (SQLException e) {
				// Print out the exception string
				println(e.toString());
			}
		}
		/*
		* It's always a good practice to close a Statement before
		* forgetting it. This applies to PreparedStatement and  
		* CallableStatement and Connection instances as well.
		*/
		stmt.close();

		println();
		println(query[3]);
		/*
		* The last query needs a PreparedStatement instance
		*/
		PreparedStatement prep = conn.prepareStatement(query[3]);
		// Set the parameter of the statement to 300
		prep.setShort(1, (short)300);
		// Execute the query and get back the ResultSet
		rs = prep.executeQuery();
		// Print out the query result
		printQuery(rs);
		// Close the statement
		prep.close();
		// Close the connection
		conn.close();
		status.setText("Done.");
	  }
	  catch (Throwable e) {
		// Print the exception string in the status line
		status.setText(e.toString());
	  }
	}

	/*
	* This method automatically fill the proxy server box using 
	* properties in the ProxyProperties class instance pxProp.
	*/
	private void setProxyHostBox() {
		String host, port;
		if (pxType.getSelectedIndex() == 2) {
			host = "https.proxyHost";
			port = "https.proxyPort";
		}
		else {
			host = "socksProxyHost";
			port = "socksProxyPort";
		}
		host = pxProp.getProperty(host);
		port = pxProp.getProperty(port);
		if (host != null && port != null)
			host = host + ':' + port;
		pxHost.setText(host);
	}

	public void actionPerformed(ActionEvent e) {
		// Capture the mouse click of the Run button
		if (e.getSource() == runButton) {
			// Clear the text window
			textWin.setText("");
			run();
		}
	}

	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == proxy) {
			boolean on = proxy.getState();
			pxHost.setEnabled(on); pxType.setEnabled(on);
			lb_uid.setEnabled(on); px_uid.setEnabled(on);
			lb_pwd.setEnabled(on); px_pwd.setEnabled(on);
			if (on)
				setProxyHostBox();
		}
		else if (e.getSource() == pxType) {
			setProxyHostBox();
		}
	}

	private static String line = "------------------------------";

	void printQuery(ResultSet rs) throws SQLException {
		// Get the ResultSetMetaData
		ResultSetMetaData md = rs.getMetaData();
		String s;
		int i, nCol = md.getColumnCount();
		// Print out the name of each returned column
		for (i = 1; i <= nCol; ++i)
			print(md.getColumnName(i) + " ");
		println();
		// Print a dividing line between column and result
		for (i = 1; i <= nCol; ++i) {
			s = md.getColumnName(i);
			// Assume line is long enough for all columns
			print(line.substring(0, s.length()) + " ");
		}
		println(); // Start a new line
		/*
		* Loop through each returned row in the ResultSet
		*/
		while (rs.next()) {
			/*
			* Loop through each column of a row
			*/
			for (i = 1; i <= nCol; ++i) {
				// Get the value of the column in String
				s = rs.getString(i);
				if (s == null)
					s = "<null>";
				// Print out the column value plus a space
				print(s.trim() + " ");
			}
			println(); // Start a new line
		}
	}

	/*
	* BorderLayout/GridLayout made easy
	*/
	public static Panel newPanel(
			Component west,
			Component north, Component center, Component south,
			Component east) {
		Panel pan = new Panel();
		if (center == null) {
			if (north == null && south == null) {
				pan.setLayout(new GridLayout(1,2));
				pan.add("1", west);
				pan.add("2", east);
			}
			else if (west == null && east == null) {
				pan.setLayout(new GridLayout(2,1));
				pan.add("1", north);
				pan.add("2", south);
			}
			else {
				pan.setLayout(new GridLayout(2,2));
				pan.add("1", west);
				pan.add("2", north);
				pan.add("3", east);
				pan.add("4", south);
			}
			return pan;
		}
		pan.setLayout(new BorderLayout());
		if (north != null) pan.add("North", north);
		if (south != null) pan.add("South", south);
		if (center!= null) pan.add("Center",center);
		if (east  != null) pan.add("East",  east);
		if (west  != null) pan.add("West",  west);
		return pan;
	}

	/*
	* Extract the origin of the running applet
	*/
	String getHost() {
		URL url = getCodeBase();
		int port = url.getPort();
		return url.getHost() + ':' + (port > 0 ? port : 80);
	}

	/*
	* For less typing and better readability
	*/
	public void print(String s) {
		textWin.append(s);
	}
	public void println(String s) {
		textWin.append(s + '\n');
	}
	public void println() {
		textWin.append("\n");
	}

}
