/*
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
* ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
* PARTICULAR PURPOSE.
*
* Copyright (C) 1996-2003  IDS Software.  All Rights Reserved.
*
*  Module:	IDSTest.java
*
*  Comment:	IDS JDBC Driver Test Program.
*
*		This program is intended to be run on the local machine
*		where IDS Server is installed. To see a demonstration on 
*		how to run an JDBC enabled applet from a remote browser,
*		please refer to the "AppTest.java" program in directory
*		"IDSServer\classes".
*
*		The queries use the ODBC data source "IDSExamples". Refer to
*		Section 1.2.2 of the IDS User's Guide for its installation.
*
*		You need to add the "IDSServer\classes" directory to the 
*		CLASSPATH environment before execution.
*
* [Histroy]
* 01-03-03  Added the use of dbShield and command line parameter as URL.
* 08-30-99  Removed the meaningless nativeSQL() call.
* 02-16-99  Copied printQuery() method from AppTest.java
* 03-07-97  Changed default to "localhost" and deleted test via CGI.
*/

import java.sql.*;

public class IDSTest {

	static String url = "jdbc:ids://localhost:12/conn?dsn='IDSExamples'";

	static String query[] = {
		"<sql name=departments>", // dbShield query
		"select * from courses",  // Matches query <sql name=courses>
		"drop table course",      // dbShield will deny this
		"<sql name=my_courses>",  // dbShield query
	};

	public static void main(String[] args) throws Exception {

		if (args.length > 0)
			url = args[0];

		Class.forName("ids.sql.IDSDriver").newInstance();
		Connection conn = DriverManager.getConnection(url);
		ResultSet rs;

		int n;
		Statement stmt = conn.createStatement();
		for (n = 0; n < 3; ++n) {
			try {
				println();
				println(query[n]);
				rs = stmt.executeQuery(query[n]);
				printQuery(rs);
			} catch (SQLException e) {
				println(e.toString());
			}
		}
		stmt.close();

		println();
		println(query[3]);
		PreparedStatement prep = conn.prepareStatement(query[3]);
		prep.setShort(1, (short)300);
		rs = prep.executeQuery();
		printQuery(rs);
		prep.close();
		conn.close();
	}

	private static String line = "------------------------------";

	static void printQuery(ResultSet rs) throws SQLException {
		// Get the ResultSetMetaData
		ResultSetMetaData md = rs.getMetaData();
		String s;
		int i, nCol = md.getColumnCount();
		// Print out the name of each returned column
		for (i = 1; i <= nCol; ++i)
			print(md.getColumnName(i) + " ");
		println();
		// Print a dividing line between column and result
		for (i = 1; i <= nCol; ++i) {
			s = md.getColumnName(i);
			// Assume line is long enough for all columns
			print(line.substring(0, s.length()) + " ");
		}
		println(); // Start a new line
		/*
		* Loop through each returned row in the ResultSet
		*/
		while (rs.next()) {
			/*
			* Loop through each column of a row
			*/
			for (i = 1; i <= nCol; ++i) {
				// Get the value of the column in String
				s = rs.getString(i);
				if (s == null)
					s = "<null>";
				// Print out the column value plus a space
				print(s.trim() + " ");
			}
			println(); // Start a new line
		}
	}

	static void print(String s) {
		System.out.print(s);
	}
	static void println(String s) {
		System.out.println(s);
	}
	static void println() {
		System.out.println();
	}
}
