/*
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
* ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
* PARTICULAR PURPOSE.
*
* Copyright (C) 1997-1999  IDS Software.  All Rights Reserved.
*
*  Module:  ProxyProperties.java
*
*  Comment: Proxy Properties Class
*
*    This class, when instanciated, contains the following set
*    of client proxy server setting of the Web browser.  It works 
*    for Internet Explorer 3.02 or higher, Netscape Communicator 
*    4.0 or higher, and the HotJava Browser.
*
*		"socksProxyHost"
*		"socksProxyPort"
*		"http.proxyHost"
*		"http.proxyPort"
*		"https.proxyHost"
*		"https.proxyPort"
*		"ftpProxyHost"
*		"ftpProxyPort"
*
*    Since it access privilege information of the client machine
*    such as System Properties, local file and Win32 registry,
*    it can only be used in stand-alone Java applications or Java
*    applets with digital signature.  It cannot be used in regular
*    Java applets.
*
*    Although it uses some Microsoft and Netscape specific Java
*    classes, it can be run in all Java VM, because the try/catch 
*    blocks handles all exceptions including ClassNotFoundException.
*
*    However, recompiling it using JDK might take some doing, because
*    it requires the present of 3 Microsoft and Netscape packages.
*    You can download the Netscape package from:
*
*        http://developer.netscape.com/docs/manuals/signedobj/capsapi_classes.zip
*
*    The two Microsoft packages are in %WINDIR%\Java\Class\classes.zip
*    If you don't have this giant ZIP file, you need to install the
*    Microsoft SDK for Java and run the following program:
*
*        clspack -auto
*
*    Recompiling this class is not recommended.
*/

import java.io.*;
import java.util.*;
import netscape.security.*;
import com.ms.security.*;
import com.ms.lang.*;

public class ProxyProperties extends Properties {

	public static String socksProxyHost  = "socksProxyHost";
	public static String socksProxyPort  = "socksProxyPort";
	public static String http_proxyHost  = "http.proxyHost";
	public static String http_proxyPort  = "http.proxyPort";
	public static String https_proxyHost = "https.proxyHost";
	public static String https_proxyPort = "https.proxyPort";
	public static String ftpProxyHost    = "ftpProxyHost";
	public static String ftpProxyPort    = "ftpProxyPort";

	public ProxyProperties() {
		/*
		* Set all proxy related System properties
		*/
		setBrowserProxyProperties();
		try {
			PrivilegeManager.enablePrivilege("UniversalPropertyRead");
			/*
			* Copy all proxy properties to this instance
			*/
			String s;
			// Set SOCKS proxy properties
			s = System.getProperty(socksProxyHost);
			if (s != null)     put(socksProxyHost, s);
			s = System.getProperty(socksProxyPort);
			if (s != null)     put(socksProxyPort, s);
			// Set HTTP proxy properties
			s = System.getProperty(http_proxyHost);
			if (s != null)     put(http_proxyHost, s);
			s = System.getProperty(http_proxyPort);
			if (s != null)     put(http_proxyPort, s);
			// Set HTTPS proxy properties
			s = System.getProperty(https_proxyHost);
			if (s != null)     put(https_proxyHost, s);
			s = System.getProperty(https_proxyPort);
			if (s != null)     put(https_proxyPort, s);
			// Set FTP proxy properties
			s = System.getProperty(ftpProxyHost);
			if (s != null)     put(ftpProxyHost, s);
			s = System.getProperty(ftpProxyPort);
			if (s != null)     put(ftpProxyPort, s);
		}
		catch (Throwable e) {}
	}

	public static void setBrowserProxyProperties() {
		String s = System.getProperty("java.vendor");
		if (s.startsWith("Netscape"))
			setNavigatorProxyProperties();
		else if (s.startsWith("Microsoft")) {
		  try {
			PolicyEngine.assertPermission(PermissionID.REGISTRY);
		  }
		  catch (Throwable e) {}
		  try {
			Properties sys = System.getProperties();
			RegKey key = new RegKey(
				RegKey.getRootKey(RegKey.USER_ROOT),
				"Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings",
				RegKey.KEYOPEN_READ
			);
			byte[] b = key.getBinaryValue("ProxyEnable");
			if (b == null || b.length < 4 || (b[0]|b[3]) == 0)
				return;
			/*
			* Parse the ProxyServer registry value
			* e.g. https=xyz:80;socks=abc:1080
			*/
			s = key.getStringValue("ProxyServer");
			int i, k, n, max = s.length();
			String h, p, t;
			for (n = 0; n < max; n = k + 1) {
				i = s.indexOf('=', n);
				if (i < 0)
					t = null; // Use the same proxy for all protocols
				else {
					t = s.substring(n, i);
					n = i + 1;
				}
				i = s.indexOf(':', n); if (i < 0) i = max;
				k = s.indexOf(';', n); if (k < 0) k = max;
				h = s.substring(n, i); // Host name
				if (i < k) ++i;
				p = s.substring(i, k); // Port number

				// Set SOCKS proxy properties
				if (t != null && t.equals("socks")) {
					sys.put(socksProxyHost, h);
					sys.put(socksProxyPort, p);
				}
				// Set HTTP proxy properties
				if (t == null || t.equals("http")) {
					sys.put(http_proxyHost, h);
					sys.put(http_proxyPort, p);
				}
				// Set HTTPS proxy properties
				if (t == null || t.equals("https")) {
					sys.put(https_proxyHost, h);
					sys.put(https_proxyPort, p);
				}
				// Set FTP proxy properties
				if (t == null || t.equals("ftp")) {
					sys.put(ftpProxyHost, h);
					sys.put(ftpProxyPort, p);
				}
			}
			// Make it permanent
			System.setProperties(sys);
		  }
		  catch (Throwable e) {}
		}
	}

	public static void setNavigatorProxyProperties() {
	  try {
		PrivilegeManager.enablePrivilege("Debugger");
		String s = System.getProperty("network.proxy.type");
		if (s != null)
			return;
		/*
		* Search prefs.js for proxy settings
		*/
		s = System.getProperty("user.home");
		int i, j = s.length();
		if (s.charAt(j-1) != File.separatorChar)
			s += File.separator;
		File f = new File(s + "prefs.js");
		if (!f.exists())
			f = new File(s + "preferences.js");
		BufferedReader in = new BufferedReader(new FileReader(f));
		Properties pref = new Properties();
		String name;
		// Read all preferences into pref
		while ((s = in.readLine()) != null) {
			if (s.indexOf("user_pref") < 0)
				continue;
			// Parse user_pref("name", "value");
			//    or user_pref("name", 21);
			i = s.indexOf('"') + 1;
			if (i < 1) continue;
			j = s.indexOf('"', i);
			if (j < 0) continue;
			name = s.substring(i, j);
			//
			i = s.indexOf(',', j) + 1;
			if (i < 1) continue;
			j = s.lastIndexOf(')');
			if (j < i) continue;
			s = s.substring(i, j).trim();
			i = s.length() - 1;
			if (i > 0 && s.charAt(0) == '"' && s.charAt(i) == '"')
				s = s.substring(1, i);
			pref.put(name, s);
		}
		// Get all system properties
		Properties sys = System.getProperties();
		/*
		* network.proxy.type:
		*    null  Direct connection (no proxy)
		*    1     Manual proxy configuration
		*    2     Automatic proxy configuration
		*/
		s = pref.getProperty("network.proxy.type");
		if (s == null) s = "0";
		sys.put("network.proxy.type", s);
		if (s.equals("1")) {
			// Set SOCKS proxy properties
			s = pref.getProperty("network.hosts.socks_server");
			if (s != null) sys.put(socksProxyHost, s);
			s = pref.getProperty("network.hosts.socks_serverport");
			if (s != null) sys.put(socksProxyPort, s);
			// Set HTTP proxy properties
			s = pref.getProperty("network.proxy.http");
			if (s != null) sys.put(http_proxyHost, s);
			s = pref.getProperty("network.proxy.http_port");
			if (s != null) sys.put(http_proxyPort, s);
			// Set HTTPS proxy properties
			s = pref.getProperty("network.proxy.ssl");
			if (s != null) sys.put(https_proxyHost, s);
			s = pref.getProperty("network.proxy.ssl_port");
			if (s != null) sys.put(https_proxyPort, s);
			// Set FTP proxy properties
			s = pref.getProperty("network.proxy.ftp");
			if (s != null) sys.put(ftpProxyHost, s);
			s = pref.getProperty("network.proxy.ftp_port");
			if (s != null) sys.put(ftpProxyPort, s);
		}
		// Make it permanent
		System.setProperties(sys);
	  }
	  catch (Throwable e) {}
	}
}
