/*
* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
* ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
* PARTICULAR PURPOSE.
*
* Copyright (C) 1997-2003  IDS Software.  All Rights Reserved.
*
*  Module:	SecureJDBC.java
*
*  Comment:	Secure JDBC Applet Test Program
*
*		This applet automatically extracts the hostname (or address)
*		of the IDS Server before running the test queries. You 
*		don't need to modify and recompile the program at all.
*
*		To run the SecureJDBC applet from a remote browser, type in 
*		the following URL:
*
*			http://server1.bogus.com:12/SecureJDBC.htm
*
*		If you are running the applet locally or your computer has 
*		no network connection, type in the following URL:
*
*			http://127.0.0.1:12/SecureJDBC.htm
*
* [History]
* 01-03-03  Added the use of dbShield.
* 01-02-02  Changed to use jdk 1.1. No longer works in 3.x browsers.
* 08-30-99  Removed the meaningless nativeSQL() call.
* 12-10-97  Modified for Version 2.5 to use DH_anon_WITH_DES40_CBC_SHA
* 07-29-97  Modified from AppTest.java with a SSL request (ssl=1)
* 07-25-97  Added getHost() method to automatically extract hostname.
* 05-12-97  Changed into an interactive applet to obtain hostname.
* 03-07-97  Changed data source and queries to 'IDSExamples'
* 02-25-97  Added exception display to the catch block of method run().
* 02-12-97  Modified to bypass DriverManager to avoid problems with IE.
*
*/

import java.util.Properties;
import java.net.URL;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.sql.*;

public class SecureJDBC extends Applet implements Runnable, ActionListener {

	static String signature = "jdbc:ids://";
	static String conn_etc  = "/conn?dsn=IDSExamples&ssl=1";

	static String query[] = {
		"<sql name=departments>", // dbShield query
		"select * from courses",  // Matches query <sql name=courses>
		"drop table course",      // dbShield will deny this
		"<sql name=my_courses>",  // dbShield query
	};

	private TextArea textWin = null;
	private Thread theThread = null;
	private TextField status = new TextField("Please wait...");
	private Panel        top = new Panel();
	private TextField   host = new TextField();
	private Label     prompt = new Label("Enter IDS Server address: ");
	private Button runButton = new Button(" Run ");
	private boolean firstRun = true;
   
	public void init() { // Applet initialization
		super.init();
		if (textWin == null) {
			// Create a non-proportional font
			Font fn = new Font("Courier", Font.PLAIN, 12);
			// Create the output text window
			textWin = new TextArea();
			// Assign the fixed font to the output window
			textWin.setFont(fn);
			// Assign a layout manager for the top group
			top.setLayout(new BorderLayout());
			// Add components into the top group
			top.add("West", prompt);
			top.add("Center", host);
			top.add("East", runButton);
			// Assign a layout manager for the main group
			setLayout(new BorderLayout());
			// Add components into the main group
			add("North", top);
			add("Center", textWin);
			add("South", status);
			runButton.addActionListener(this);
		}
	}

	public void start() {
		textWin.setText("");
		if (theThread == null) {
			theThread = new Thread(this);
			theThread.start();
		}
	}
	public void stop() {
		if ((theThread != null) && theThread.isAlive()) 
			theThread.stop();
		theThread = null;
	}
    
	public void run() {

	  try {
		/*
		* Let's first compose the connection URL
		*/
		String url  = signature;     // Always starts with this
		String addr = host.getText();
		if (addr.length() == 0) {    // Do nothing if no address
			status.setText("Ready");
			if (firstRun) {
				// Fill in the host name field
				host.setText(getHost());
				firstRun = false;
			}
			return;
		}
		url += addr + conn_etc;      // Compose the connection URL
		/*
		* Create an IDS JDBC Driver of the ids.sql package
		*/
		status.setText("Initializing IDS JDBC Driver...");
		Driver drv = new ids.sql.IDSDriver();
		/*
		* Pick a preferred cipher-suite instead of the default.
		* Normally, you should use the default cipher-suites,
		* which means by simply calling drv.connect(url, null);
		* Refer to Section 5.3.5 of the User's Guide for detail.
		*/
		Properties info = new Properties();
		/*
		* WARNING: Do not use any DH_anon_WITH_?_?_? cipher-suites
		* without fully understand the risks of "man in the middle"
		* attack.  It is use here so that the example does not 
		* depend on the a default public/private key pair.
		* Refer to Section 5.5 of the User's Guide for detail.
		*
		* Normally, you should use the default cipher-suites,
		* which means by simply calling drv.connect(url, null);
		*/
		info.put("CipherSuite_1", "DH_anon_WITH_DES40_CBC_SHA");
		/*
		* Connect to the IDS Server using the composed URL
		*/
		status.setText("Connecting to IDS Server...");
		Connection conn = drv.connect(url, info);

		ResultSet rs;
		int n;
		/*
		* Create a dummy Statement class instance
		*/
		status.setText("Creating SQL statement object...");
		Statement stmt = conn.createStatement();
		/*
		* Loop through the first three queries in the query[] array.
		* Note that the same Statement instance is re-used during
		* each iteration.
		*/
		status.setText("Executing SQL statements...");
		for (n = 0; n < 3; ++n) {
			try {
				println();
				println(query[n]);
				// Execute the predefined query in the array
				rs = stmt.executeQuery(query[n]);
				// Print out the query result
				printQuery(rs);
			} catch (SQLException e) {
				// Print out the exception string
				println(e.toString());
			}
		}
		/*
		* It's always a good practice to close a Statement before
		* forgetting it. This applies to PreparedStatement and  
		* CallableStatement and Connection instances as well.
		*/
		stmt.close();

		println();
		println(query[3]);
		/*
		* The last query needs a PreparedStatement instance
		*/
		PreparedStatement prep = conn.prepareStatement(query[3]);
		// Set the parameter of the statement to 300
		prep.setShort(1, (short)300);
		// Execute the query and get back the ResultSet
		rs = prep.executeQuery();
		// Print out the query result
		printQuery(rs);
		// Close the statement
		prep.close();
		// Close the connection
		conn.close();
		status.setText("Done.");
	  }
	  catch (Throwable e) {
		// Print the exception string in the status line
		status.setText(e.toString());
	  }
	}

	public void actionPerformed(ActionEvent e) {
		// Capture the mouse click of the Run button
		if (e.getSource() == runButton) {
			// Clear the text window
			textWin.setText("");
			run();
		}
	}

	private static String line = "------------------------------";

	void printQuery(ResultSet rs) throws SQLException {
		// Get the ResultSetMetaData
		ResultSetMetaData md = rs.getMetaData();
		String s;
		int i, nCol = md.getColumnCount();
		// Print out the name of each returned column
		for (i = 1; i <= nCol; ++i)
			print(md.getColumnName(i) + " ");
		println();
		// Print a dividing line between column and result
		for (i = 1; i <= nCol; ++i) {
			s = md.getColumnName(i);
			// Assume line is long enough for all columns
			print(line.substring(0, s.length()) + " ");
		}
		println(); // Start a new line
		/*
		* Loop through each returned row in the ResultSet
		*/
		while (rs.next()) {
			/*
			* Loop through each column of a row
			*/
			for (i = 1; i <= nCol; ++i) {
				// Get the value of the column in String
				s = rs.getString(i);
				if (s == null)
					s = "<null>";
				// Print out the column value plus a space
				print(s.trim() + " ");
			}
			println(); // Start a new line
		}
	}

	/*
	* Extract the origin of the running applet
	*/
	String getHost() {
		URL url = getCodeBase();
		int port = url.getPort();
		return url.getHost() + ':' + (port > 0 ? port : 80);
	}

	/*
	* For less typing and better readability
	*/
	public void print(String s) {
		textWin.append(s);
	}
	public void println(String s) {
		textWin.append(s + '\n');
	}
	public void println() {
		textWin.append("\n");
	}

}
