/*
* Copyright (C) 1996-2002  IDS Software.  All Rights Reserved.
*
* idss_api.h -- IDS Server API module header
*
* This software is the confidential and proprietary information of 
* IDS Software ("Confidential Information").  You shall not
* disclose such Confidential Information and shall use it only in
* accordance with the terms of the license agreement you entered into
* with IDS Software.
* 
* IDS SOFTWARE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY 
* OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
* PURPOSE, OR NON-INFRINGEMENT. IDS SOFTWARE SHALL NOT BE LIABLE FOR ANY 
* DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR 
* DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
*/

#ifndef IDSS_API_H
#define IDSS_API_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef WINVER
#define _cdecl
#endif

/* Constants for IDSServerInfo(int info,,) */
#define IDSS_VERSION	0
#define IDSS_VER_MASK	1
#define IDSS_OPT_MASK	2
#define IDSS_HOME_DIR	3
#define IDSS_SAFE_EXIT	4

/* Constants for IDSS_VER_MASK return */
#define IDSS_VER_LITE	1
#define IDSS_VER_EXPORT	2
#define IDSS_VER_BETA	0x40
#define IDSS_VER_EVAL	0x80

/* Function pointer types */
typedef int  (_cdecl *F_IDSServerInfo)(int info, char *buf, int max);
typedef int  (_cdecl *F_IDSServerInit)(int argc, char *argv[]);
typedef void (_cdecl *F_IDSServerRun)(void *);
typedef int  (_cdecl *F_IDSServerStart)(int argc, char *argv[]);
typedef void (_cdecl *F_IDSServerStop)(int bWait);
typedef void (_cdecl *F_IDSServerPause)();
typedef void (_cdecl *F_IDSServerContinue)();

/*
* int IDSServerInfo(int info, char *buf, int max);
*
*	Returns information about the IDS Server.
*
* Parameters:
*	info	Information constant
*	buf 	Buffer to receive string info
*	max 	Maximum size of buf
*
* For string info, if max is less than the size of the
* string to be returned, the function returns the size
* of the string (including the null terminator) without 
* copying anything into buf. Otherwise, it returns 0,
* and the null terminated string is copied into buf.
*
* info				Returns
* ------------		----------------------------------------
* IDSS_VERSION		Version string in the form of n.n.n
*
* IDSS_VER_MASK		32-bit mask of version in the form of
*               	0xMMNNRRTT, where MM, NN, RR are major,
*               	minor and revision number. TT contains
*               	the bit mask of constant IDSS_VER_xxx
*
* IDSS_OPT_MASK		Reserved
*
* IDSS_HOME_DIR 	Returns the home directory string of the
*               	IDS Server installed in this system. The
*               	string always ends with a '/'.
*
* IDSS_SAFE_EXIT	Returns 1 if no active connection remains,
*               	and it is safe to unload the API DLL.
*/
int _cdecl IDSServerInfo(int info, char *buf, int max);

/*
* int IDSServerInit(int argc, char *argv[]);
*
*	Initializes the IDS Server. This function does not run
*	the IDS Server.  The calling module should later call
*	API function IDSServerRun(void *) to run the server.
*
* Parameters:
*	argc	Number of arguements
*	argv	Array of arguement string pointers
*
* Returns:
*	0   	Success
*	1   	Failed to find the "idss.ini" file
*	2   	Failed to create a socket
*	3   	Failed to bind to the socket
*	4   	Failed to listen to the socket
*	5   	Busy (this API only supports 1 instance of IDS Server)
*	-1  	WinSock API (1.1 or higher) failure
*
* This resembles the standard C function main(int, char**).
* It assumes argv[0] is the home directory of the IDS Server.
* If argv or argv[0] is NULL, this function will call function
* IDSServerInfo(IDSS_HOME_DIR,,) to obtain the home dirctory.
* For example,
*
*		char *argv[1];
*		...
*		argv[0] = "c:\\IDSServer\\";
*		...
*		IDSServerInit(1, argv);
*
* IMPORTANT: Requirements for argv[0] are as follows:
*
*	o The path name in argv[0] must end with a slash or 
*	  backslash, or a file name in the directory.
*
*	o The directory in argv[0] must contain file "idss.ini"
*	  and at least the following sub-directory structure:
*
*		|
*		+-- Logs\
*		|
*		+-- Security\
*		|
*		+-- File\
*		      |
*		      +-- wwwroot\
*/
int _cdecl IDSServerInit(int argc, char *argv[]);

/*
* void IDSServerRun(void *);
*
*	Runs the IDS Server in the current thread. This function
*	will not return until the IDS Server is shutdown. The 
*	calling module should spawn a new thread if it does not
*	want to be blocked by this function. This function name
*	can be used as the first parameter of _beginthread().
*
*		char *argv[1];
*		...
*		argv[0] = "c:\\IDSServer\\";
*		...
*		IDSServerInit(1, argv);
*		...
*		_beginthread(IDSServerRun, 0, NULL);
*
* Parameters:
*	void *	Reserved for future use, but should be NULL.
*/
void _cdecl IDSServerRun(void *);

/*
* int IDSServerStart(int argc, char *argv[]);
*
*	Starts the IDS Server. This function returns immediately
*	after calling IDSServerInit(int, char**) and creating
*	a new thread to run IDSServerRun(void*).
*
* Parameters:
*	(See IDSServerInit)
*/
int _cdecl IDSServerStart(int argc, char *argv[]);

/*
* void IDSServerStop(int bWait);
*
*	Stops the IDS Server
*
* Parameters:
*	bWait	0: Function returns immediately
*	     	1: Waits until IDS Server is shutdown
*/
void _cdecl IDSServerStop(int bWait);

/*
* Suspend the operation of the IDS Server
*/
void _cdecl IDSServerPause();

/*
* Resumes the operation of the IDS Server
*/
void _cdecl IDSServerContinue();

#ifdef __cplusplus
}
#endif

#endif
